/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: IBM Corporation - initial API and implementation
 ******************************************************************************/
package org.eclipse.equinox.p2.tests.generator;

import org.eclipse.equinox.internal.p2.metadata.generator.features.FeatureParser;
import org.eclipse.equinox.internal.provisional.p2.core.VersionRange;
import org.eclipse.equinox.internal.provisional.p2.metadata.IInstallableUnit;
import org.eclipse.equinox.internal.provisional.p2.metadata.IInstallableUnitPatch;
import org.eclipse.equinox.internal.provisional.p2.metadata.generator.Feature;
import org.eclipse.equinox.internal.provisional.p2.metadata.generator.MetadataGeneratorHelper;
import org.eclipse.equinox.p2.tests.AbstractProvisioningTest;

public class PatchIUGeneration extends AbstractProvisioningTest {

	public void testGeneratedIU() {
		FeatureParser parser = new FeatureParser();
		Feature feature = parser.parse(getTestData("org.eclipse.jdt.core.feature.patch", "testData/org.eclipse.jdt.3.2.1.patch_1.0.0.jar"));
		if (feature == null)
			fail();
		IInstallableUnit featureIU = MetadataGeneratorHelper.createFeatureJarIU(feature, true, null);
		IInstallableUnitPatch patchIU = (IInstallableUnitPatch) MetadataGeneratorHelper.createGroupIU(feature, featureIU, null, true);

		//Check id
		assertEquals(patchIU.getId(), "org.eclipse.jdt.3.2.1.patch.feature.group");

		//Check applicability scope
		assertEquals(IInstallableUnit.NAMESPACE_IU_ID, patchIU.getApplicabilityScope()[0][0].getNamespace());
		assertEquals("org.eclipse.jdt.feature.group", patchIU.getApplicabilityScope()[0][0].getName());
		assertEquals(new VersionRange("[3.2.1.r321_v20060905-R4CM1Znkvre9wC-,3.2.1.r321_v20060905-R4CM1Znkvre9wC-]"), patchIU.getApplicabilityScope()[0][0].getRange());

		assertEquals("org.eclipse.jdt.core", patchIU.getRequirementsChange()[0].applyOn().getName());
		assertEquals(VersionRange.emptyRange, patchIU.getRequirementsChange()[0].applyOn().getRange());
		assertEquals("org.eclipse.jdt.core", patchIU.getRequirementsChange()[0].newValue().getName());
		assertEquals(new VersionRange("[3.2.2,3.2.2]"), patchIU.getRequirementsChange()[0].newValue().getRange());
		assertEquals(Boolean.TRUE.toString(), patchIU.getProperty(IInstallableUnit.PROP_TYPE_PATCH));
		assertEquals(1, patchIU.getRequiredCapabilities().length);
		assertEquals(featureIU.getId(), patchIU.getRequiredCapabilities()[0].getName());
		assertEquals("org.eclipse.jdt.feature.group", patchIU.getLifeCycle().getName());
		assertFalse(patchIU.getLifeCycle().isGreedy());
		assertFalse(patchIU.getLifeCycle().isOptional());
	}
}
