/*******************************************************************************
 * Copyright (c) 2009 Cloudsmith Inc. and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Cloudsmith Inc. - initial API and implementation
 *******************************************************************************/

package org.eclipse.equinox.p2.tests.omniVersion;

import org.eclipse.equinox.internal.provisional.p2.core.Version;
import org.eclipse.equinox.internal.provisional.p2.core.VersionVector;

/**
 * Tests the OmniVersion raw version format.
 *
 */
public class RawVersionTest extends VersionTesting {

	public void testBasicParsing() {

		// should parse without exception
		assertNotNull(Version.parseVersion("raw:1"));
		assertNotNull(Version.parseVersion("raw:1.0"));
		assertNotNull(Version.parseVersion("raw:1.0.0"));
		assertNotNull(Version.parseVersion("raw:1.0.0.9"));
		assertNotNull(Version.parseVersion("raw:1.0.0.'r12345'"));
		assertNotNull(Version.parseVersion("raw:1.0.0.'r12345.hello'"));

		assertNotNull(Version.parseVersion("raw:1.0.m"));
		assertNotNull(Version.parseVersion("raw:1.0.M"));

		assertNotNull(Version.parseVersion("raw:1.0.M"));
		assertNotNull(Version.parseVersion("raw:1.0.-M"));

	}

	public void testSerialize() {
		Version v = null;
		// should parse without exception
		assertNotNull(v = Version.parseVersion("raw:1"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0.0"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0.0.9"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0.0.'r12345'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0.0.'r12345.hello'"));
		assertSerialized(v);

		assertNotNull(v = Version.parseVersion("raw:1.0.m"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0.M"));
		assertSerialized(v);

		assertNotNull(v = Version.parseVersion("raw:1.0.M"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0.-M"));
		assertSerialized(v);

		assertNotNull(v = Version.parseVersion("raw:0"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:0.1.2.3.4.5.6.7.8.9"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:0.-1.-2.-3.-4.-5.-6.-7.-8.-9"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:123456789.-1234567890"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:123456789.-1234567890"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:m"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:M"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:-M"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.m"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.M"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.-M"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:'a'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:\"a\""));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:'ab'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:'abcdefghijklmnopqrstuvwxyz0123456789'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:'-_!\"#$%&/()=?+*;,:.'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:\"'\""));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:'\"'"));
		assertSerialized(v);

	}

	public void testVersionString() {
		Version v = null;
		String s = null;
		// should parse without exception
		assertNotNull(v = Version.parseVersion(s = "raw:1"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0.0"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0.0.9"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0.0.'r12345'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0.0.'r12345.hello'"));
		assertEquals(s, v.toString());

		assertNotNull(v = Version.parseVersion(s = "raw:1.0.m"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0.M"));
		assertEquals(s, v.toString());

		assertNotNull(v = Version.parseVersion(s = "raw:1.0.M"));
		assertEquals(s, v.toString());

		// -M is normalized
		assertNotNull(v = Version.parseVersion("raw:1.0.-M"));
		s = "raw:1.0";
		assertEquals(s, v.toString());

		assertNotNull(v = Version.parseVersion(s = "raw:0"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:0.1.2.3.4.5.6.7.8.9"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:0.-1.-2.-3.-4.-5.-6.-7.-8.-9"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:123456789.-1234567890"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:123456789.-1234567890"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:m"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:M"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:-M"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.m"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.M"));
		assertEquals(s, v.toString());
		// -M is normalized
		assertNotNull(v = Version.parseVersion("raw:1.-M"));
		s = "raw:1";
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:'a'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:\"a\""));
		// " is normalized to '
		s = "raw:'a'";
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:'ab'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:'abcdefghijklmnopqrstuvwxyz0123456789'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:'-_!\"#$%&/()=?+*;,:.'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:\"'\""));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:'\"'"));
		assertEquals(s, v.toString());

	}

	public void testIntegerParsing() {

		// should parse without exception
		Version v = Version.parseVersion("raw:0");
		assertNotNull(v);
		assertEquals(v.getSegment(0), Integer.valueOf(0));

		// single digits
		v = Version.parseVersion("raw:0.1.2.3.4.5.6.7.8.9");
		assertNotNull(v);
		for (int i = 0; i < 10; i++)
			assertEquals(v.getSegment(i), Integer.valueOf(i));

		// negative single digits
		v = Version.parseVersion("raw:0.-1.-2.-3.-4.-5.-6.-7.-8.-9");
		assertNotNull(v);
		for (int i = 0; i < 10; i++)
			assertEquals(v.getSegment(i), Integer.valueOf(-i));

		// some larger numbers
		v = Version.parseVersion("raw:123456789.-1234567890");
		assertNotNull(v);
		assertEquals(v.getSegment(0), Integer.valueOf(123456789));
		assertEquals(v.getSegment(1), Integer.valueOf(-1234567890));
	}

	public void testWhiteSpaceExceptions() {

		try {
			Version.parseVersion("raw: 0 ");
			fail("space not allowed 1");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}

		try {
			Version.parseVersion("raw:0 .1  . 'a'.   'b c d'. 4. 5. 6.   7. 8 .  9");
			fail("space not allowed 2");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}

		try {
			Version.parseVersion("raw:< 1.2.3>");
			fail("space not allowed in array 1");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
		try {
			Version.parseVersion("raw:<1.2.3 >");
			fail("space not allowed in array 2");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
		try {
			Version.parseVersion("raw:1.- 1");
			fail("Uncaught error: space between minus and number in negative");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testMaxParsing() {

		assertNotNull(Version.parseVersion("raw:m"));
		assertNotNull(Version.parseVersion("raw:M"));
		assertNotNull(Version.parseVersion("raw:-M"));

		assertNotNull(Version.parseVersion("raw:1.m"));
		assertNotNull(Version.parseVersion("raw:1.M"));
		assertNotNull(Version.parseVersion("raw:1.-M"));
	}

	public void testStringParsing() {
		Version v = Version.parseVersion("raw:'a'");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "a");

		assertNotNull(v = Version.parseVersion("raw:\"a\""));
		assertEquals(v.getSegment(0), "a");

		assertNotNull(v = Version.parseVersion("raw:'ab'"));
		assertEquals(v.getSegment(0), "ab");

		assertNotNull(v = Version.parseVersion("raw:'abcdefghijklmnopqrstuvwxyz0123456789'"));
		assertEquals(v.getSegment(0), "abcdefghijklmnopqrstuvwxyz0123456789");

		assertNotNull(v = Version.parseVersion("raw:'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'"));
		assertEquals(v.getSegment(0), "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");

		assertNotNull(v = Version.parseVersion("raw:'-_!\"#$%&/()=?+*;,:.'"));
		assertEquals(v.getSegment(0), "-_!\"#$%&/()=?+*;,:.");

		assertNotNull(v = Version.parseVersion("raw:\"'\""));
		assertEquals(v.getSegment(0), "'");

		assertNotNull(v = Version.parseVersion("raw:'\"'"));
		assertEquals(v.getSegment(0), "\"");
	}

	public void testEmptyStringParsing() {
		Version v = Version.parseVersion("raw:''");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "");

		v = Version.parseVersion("raw:\"\"");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "");
	}

	public void testStringConcatenation() {
		Version v = Version.parseVersion("raw:'ab''cd'");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "abcd");

		v = Version.parseVersion("raw:'ab'\"cd\"");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "abcd");

		v = Version.parseVersion("raw:\"ab\"\"cd\"");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "abcd");
	}

	public void testStringToString() {
		// string is normalized
		assertEquals("raw:'abcd'", Version.parseVersion("raw:'ab''cd'").toString());

		// string is normalized
		assertEquals("raw:'abcd'", Version.parseVersion("raw:'ab'\"cd\"").toString());

		// string is normalized
		assertEquals("raw:'abcd'", Version.parseVersion("raw:\"ab\"\"cd\"").toString());

		assertEquals("raw:\"'\"", Version.parseVersion("raw:\"'\"").toString());

		assertEquals("raw:'\"'", Version.parseVersion("raw:'\"'").toString());

		// quotes are normalized - default ' should be used until " is needed and vice versa.
		assertEquals("raw:'abc\"xxx\"and '\"'yyy'\"", Version.parseVersion("raw:'abc\"xxx\"'\"and 'yyy'\"").toString());

	}

	public void testArrayParsing() {
		Version v = null;
		assertNotNull(v = Version.parseVersion("raw:<1>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:1"));

		assertNotNull(v = Version.parseVersion("raw:<1.0>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:1.0"));

		assertNotNull(v = Version.parseVersion("raw:<'a'>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:'a'"));

		assertNotNull(v = Version.parseVersion("raw:<'a'.'b'>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:'a'.'b'"));

		assertNotNull(v = Version.parseVersion("raw:<'a'.'b''c'>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:'a'.'bc'")); // with concatenation

		assertNotNull(v = Version.parseVersion("raw:<1.2.-M>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:1.2.-M"));

		assertNotNull(v = Version.parseVersion("raw:<1.2.m>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:1.2.m"));

		assertNotNull(v = Version.parseVersion("raw:<1.2.M>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:1.2.M"));

		assertNotNull(v = Version.parseVersion("raw:<<1>>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:<1>"));

		assertNotNull(v = Version.parseVersion("raw:<<1.<2>>>"));
		assertEquals(v.getSegment(0), Version.parseVersion("raw:<1.<2>>"));

	}

	public void testArraySerialize() {
		Version v = null;
		assertNotNull(v = Version.parseVersion("raw:<1>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<1.0>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<'a'>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<'a'.'b'>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<'a'.'b''c'>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<1.2.-M>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<1.2.m>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<1.2.M>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<<1>>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:<<1.<2>>>"));
		assertSerialized(v);

	}

	public void testArraytoString() {
		Version v = null;
		String s = null;
		assertNotNull(v = Version.parseVersion(s = "raw:<1>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<1.0>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<'a'>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<'a'.'b'>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<'a'.'bc'>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion("raw:<1.2.-M>"));
		s = "raw:<1.2>"; // is normalized
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<1.2.m>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<1.2.M>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<<1>>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:<<1.<2>>>"));
		assertEquals(s, v.toString());
	}

	public void testArrayOrder() {
		Version v1 = Version.parseVersion("raw:<1.0.0>");
		Version v2 = Version.parseVersion("raw:<1.1.0>");

		Version v3 = Version.parseVersion("raw:<1.0.0>.<1.0.0>");
		Version v4 = Version.parseVersion("raw:<1.0.0>.<1.0.0>.'a'");
		Version v5 = Version.parseVersion("raw:<1.0.0>.<1.0.1>");
		Version v6 = Version.parseVersion("raw:<2.0.0>");

		assertOrder(v1, v2);
		assertOrder(v3, v4);
		assertOrder(v4, v5);
		assertOrder(v5, v6);
	}

	public void testPadParsing1() {
		Version v = null;
		assertNotNull(v = Version.parseVersion("raw:1.0p0"));
		assertPad(v, "raw:0");
		assertNotNull(v = Version.parseVersion("raw:1.0p'foo'"));
		assertPad(v, "raw:'foo'");
		assertNotNull(v = Version.parseVersion("raw:1.0p<0>"));
		assertPad(v, "raw:<0>");
		assertNotNull(v = Version.parseVersion("raw:1.0p<'foo'>"));
		assertPad(v, "raw:<'foo'>");
		assertNotNull(v = Version.parseVersion("raw:1.0pm"));
		assertPad(v, "raw:m");
		assertNotNull(v = Version.parseVersion("raw:1.0pM"));
		assertPad(v, "raw:M");
		assertNotNull(v = Version.parseVersion("raw:1.0p-M"));
		assertEquals(v.getPad(), null);
		assertNotNull(v = Version.parseVersion("raw:1.0p<m>"));
		assertPad(v, "raw:<m>");
		assertNotNull(v = Version.parseVersion("raw:1.0p<M>"));
		assertPad(v, "raw:<M>");
		assertNotNull(v = Version.parseVersion("raw:1.0p<-M>"));
		assertPad(v, "raw:<-M>");
		assertNotNull(v = Version.parseVersion("raw:1.0p<1.0.0.'r12345'.m>"));
		assertPad(v, "raw:<1.0.0.'r12345'.m>");
	}

	public void testPadSerialize() {
		Version v = null;
		assertNotNull(v = Version.parseVersion("raw:1.0p0"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p'foo'"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<'foo'>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0pm"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0pM"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p-M"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<m>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<M>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<-M>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<1.0.0.'r12345'.m>"));
		assertSerialized(v);
	}

	public void testPadtoString() {
		Version v = null;
		String s = null;
		assertNotNull(v = Version.parseVersion("raw:1.0p0"));
		s = "raw:1p0"; // normalized
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p'foo'"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<'foo'>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0pm"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0pM"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion("raw:1.0p-M"));
		s = "raw:1.0"; // normalized
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<m>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<M>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<-M>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<1.0.0.'r12345'.m>"));
		assertEquals(s, v.toString());
	}

	/**
	 * Test parsing of a pad with an array that in turn has padding.
	 */
	public void testNestedPadParsing() {
		Version v = null;
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p0>"));
		assertPad(v, "raw:<0p0>");
		assertPad(v.getPad(), "raw:0");
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p'foo'>"));
		assertPad(v, "raw:<0p'foo'>");
		assertPad(v.getPad(), "raw:'foo'");
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p<0>>"));
		assertPad(v, "raw:<0p<0>>");
		assertPad(v.getPad(), "raw:<0>");
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p<'foo'>>"));
		assertPad(v, "raw:<0p<'foo'>>");
		assertPad(v.getPad(), "raw:<'foo'>");

		assertNotNull(v = Version.parseVersion("raw:1.0p<0pm>"));
		assertPad(v, "raw:<0pm>");
		assertPad(v.getPad(), "raw:m");

		assertNotNull(v = Version.parseVersion("raw:1.0p<0pM>"));
		assertPad(v, "raw:<0pM>");
		assertPad(v.getPad(), "raw:M");

		assertNotNull(v = Version.parseVersion("raw:1.0p<0p-M>"));
		assertPad(v, "raw:<0p-M>");
		assertEquals(((VersionVector) v.getPad()).getPad(), null);

		assertNotNull(v = Version.parseVersion("raw:1.0p<0p<m>>"));
		assertPad(v, "raw:<0p<m>>");
		assertPad(v.getPad(), "raw:<m>");

		assertNotNull(v = Version.parseVersion("raw:1.0p<0pM>"));
		assertPad(v, "raw:<0pM>");
		assertPad(v.getPad(), "raw:M");

		assertNotNull(v = Version.parseVersion("raw:1.0p<0p-M>"));
		assertPad(v, "raw:<0p-M>");
		assertEquals(((VersionVector) v.getPad()).getPad(), null);
	}

	public void testNestedPadSerialize() {
		Version v = null;
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p0>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p'foo'>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p<0>>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p<'foo'>>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0pm>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0pM>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p-M>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p<m>>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0pM>"));
		assertSerialized(v);
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p-M>"));
		assertSerialized(v);
	}

	public void testNestedPadtoString() {
		Version v = null;
		String s = null;
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0p0>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0p'foo'>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0p<0>>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0p<'foo'>>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0pm>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0pM>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p-M>"));
		s = "raw:1.0p<0>"; // normalized
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0p<m>>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion(s = "raw:1.0p<0pM>"));
		assertEquals(s, v.toString());
		assertNotNull(v = Version.parseVersion("raw:1.0p<0p-M>"));
		s = "raw:1.0p<0>"; // normalized
		assertEquals(s, v.toString());
	}

	/**
	 * Tests that:
	 * 		1p-M < 1.0.0 < 1.0.0p0 == 1p0 < 1.1 < 1.1.1 < 1p1 == 1.1p1 < 1pM
	 */
	public void testPadOrder() {
		Version v1 = Version.parseVersion("raw:1p-M");
		Version v2 = Version.parseVersion("raw:1.0.0");
		Version v3 = Version.parseVersion("raw:1.0.0p0");
		Version v4 = Version.parseVersion("raw:1p0");
		Version v5 = Version.parseVersion("raw:1.1");
		Version v6 = Version.parseVersion("raw:1.1.1");
		Version v7 = Version.parseVersion("raw:1p1");
		Version v8 = Version.parseVersion("raw:1.1p1");
		Version v9 = Version.parseVersion("raw:1pM");

		assertOrder(v1, v2);
		assertOrder(v2, v3);
		assertEquals(v3, v4);
		assertOrder(v4, v5);
		assertOrder(v5, v6);
		assertOrder(v6, v7);
		assertEquals(v7, v8);
		assertOrder(v8, v9);
	}

	public void testPadTypeOrder() {
		Version v0 = Version.parseVersion("raw:1p-M");
		Version v1 = Version.parseVersion("raw:1p'a'");
		Version v2 = Version.parseVersion("raw:1p<0>");
		Version v3 = Version.parseVersion("raw:1.0.0");
		Version v4 = Version.parseVersion("raw:1p0");
		Version v5 = Version.parseVersion("raw:1p1");
		Version v6 = Version.parseVersion("raw:1pM");
		assertOrder(v0, v1);
		assertOrder(v1, v2);
		assertOrder(v2, v3);
		assertOrder(v3, v4);
		assertOrder(v4, v5);
		assertOrder(v5, v6);
	}

	/**
	 * Test that a / is not prematurely taken as the separator between raw and original when in a string
	 */
	public void testOriginalTerminator() {
		Version v = Version.parseVersion("raw:'/'");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "/");

		v = Version.parseVersion("raw:\"/\"");
		assertNotNull(v);
		assertEquals(v.getSegment(0), "/");
	}

	public void testEmptyInput() {
		// should parse with exception 
		try {
			Version.parseVersion("raw:");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testNewLine() {
		// should parse with exception 
		try {
			Version.parseVersion("raw:1.'\n'.2");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testSpaceInInt() {
		// should parse with exception 
		try {
			Version.parseVersion("raw:1 2.2");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testFloatingPointl() {
		// should parse with exception 
		try {
			Version.parseVersion("raw:1,2.2");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testScientific() {
		// should parse with exception 
		try {
			Version.parseVersion("raw:1E3");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testHex() {
		// should parse with exception 
		try {
			Version.parseVersion("raw:0xABCD");
			fail("Uncaught error: hexadecimal not allowed");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedSingleQuoteRight() {
		try {
			Version.parseVersion("raw:'unbalanced");
			fail("Uncaught error: unbalanced sngle quote");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testMixedQuotes1() {
		try {
			Version.parseVersion("raw:1.\"unbalanced'.10");
			fail("Uncaught error: mixed quotes");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testMixedQuotes2() {
		try {
			Version.parseVersion("raw:1.'unbalanced\".10");
			fail("Uncaught error: mixed quotes");
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedDoubleQuoteRight() {
		try {
			Version.parseVersion("raw:\"unbalanced");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedArrayRight() {
		try {
			Version.parseVersion("raw:<1.2.3");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedArrayLeft() {
		try {
			Version.parseVersion("raw:1.2.3>");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testBadDecimalInteger() {
		try {
			Version.parseVersion("raw:12af");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnquotedStringFirstValue() {
		try {
			Version.parseVersion("raw:a");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnquotedStringSecondValue() {
		try {
			Version.parseVersion("raw:1.a");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testSinglePeriod() {
		try {
			Version.parseVersion("raw:.");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testTwoPeriods() {
		try {
			Version.parseVersion("raw:..");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testThreePeriods() {
		try {
			Version.parseVersion("raw:...");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}

	}

	public void testPadNotLast() {
		try {
			Version.parseVersion("raw:p10.10");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}

	}

	public void testEmptyPad() {
		try {
			Version.parseVersion("raw:10p");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testPadWithNull() {
		try {
			Version.parseVersion("raw:10p.");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testWrongPadSeparator() {
		try {
			Version.parseVersion("raw:10.p0");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testMultiplePadElements() {
		try {
			Version.parseVersion("raw:10p1.2");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedPadElementsSQ() {
		try {
			Version.parseVersion("raw:10p'abc");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedPadElementsDQ() {
		try {
			Version.parseVersion("raw:10p\"abc");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedPadArrayElementsRight() {
		try {
			Version.parseVersion("raw:10p<10");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testUnbalancedPadArrayElementsLeft() {
		try {
			Version.parseVersion("raw:10p10>");
			fail();
		} catch (IllegalArgumentException e) {
			assertTrue(true);
		}
	}

	public void testOrder() {
		Version v1 = Version.parseVersion("raw:1");
		Version v2 = Version.parseVersion("raw:1.0");
		Version v3 = Version.parseVersion("raw:1.0.0");
		Version v4 = Version.parseVersion("raw:1.0.0.'9'");
		Version v5 = Version.parseVersion("raw:1.0.0.'r12345'");

		assertOrder(v1, v2);
		assertOrder(v2, v3);
		assertOrder(v3, v4);
		assertOrder(v4, v5);
	}

	public void testTypeOrder() {
		// '' < 'z' < m < [0} < [M] < 0 < M
		Version v0 = Version.parseVersion("raw:-M");
		Version v1 = Version.parseVersion("raw:''");
		Version v2 = Version.parseVersion("raw:'z'");
		Version v3 = Version.parseVersion("raw:m");
		Version v4 = Version.parseVersion("raw:<0>");
		Version v5 = Version.parseVersion("raw:<M>");
		Version v6 = Version.parseVersion("raw:0");
		Version v7 = Version.parseVersion("raw:M");

		assertOrder(v0, v1);
		assertOrder(v1, v2);
		assertOrder(v2, v3);
		assertOrder(v3, v4);
		assertOrder(v4, v5);
		assertOrder(v5, v6);
		assertOrder(v6, v7);
	}

	public void testTypeOrder2() {
		// '' < 'z' < m < [0] < [M] < 0 < M
		Version v0 = Version.parseVersion("raw:0.-M");
		Version v1 = Version.parseVersion("raw:0.''");
		Version v2 = Version.parseVersion("raw:0.'z'");
		Version v3 = Version.parseVersion("raw:0.m");
		Version v4 = Version.parseVersion("raw:0.<0>");
		Version v5 = Version.parseVersion("raw:0.<M>");
		Version v6 = Version.parseVersion("raw:0.0");
		Version v7 = Version.parseVersion("raw:0.M");

		assertOrder(v0, v1);
		assertOrder(v1, v2);
		assertOrder(v2, v3);
		assertOrder(v3, v4);
		assertOrder(v4, v5);
		assertOrder(v5, v6);
		assertOrder(v6, v7);
	}

	public void testShorterIsOlder() {
		Version v1 = Version.parseVersion("raw:1.0");
		Version v2 = Version.parseVersion("raw:1.0.0");
		Version v3 = Version.parseVersion("raw:1.0.0.0");

		Version v4 = Version.parseVersion("raw:'a'");
		Version v5 = Version.parseVersion("raw:'a'.'b'.'b'");
		Version v6 = Version.parseVersion("raw:'a'.'b'.'b'.'b'");

		Version v7 = Version.parseVersion("raw:<1>");
		Version v8 = Version.parseVersion("raw:<1>.<0>.<0>");
		Version v9 = Version.parseVersion("raw:<1>.<0>.<0>.<0>");

		assertOrder(v1, v2);
		assertOrder(v2, v3);

		assertOrder(v4, v5);
		assertOrder(v5, v6);

		assertOrder(v7, v8);
		assertOrder(v8, v9);
	}

	public void testNumericVersionOrder() {
		Version v1 = Version.parseVersion("1");
		Version v2 = Version.parseVersion("1.0.1");
		Version v3 = Version.parseVersion("1.1");
		Version v4 = Version.parseVersion("1.1.1");
		Version v5 = Version.parseVersion("1.1.1.-");
		Version v6 = Version.parseVersion("1.2");
		Version v7 = Version.parseVersion("2");
		Version v8 = Version.parseVersion("10.0");

		assertOrder(v1, v2);
		assertOrder(v2, v3);
		assertOrder(v3, v4);
		assertOrder(v4, v5);
		assertOrder(v5, v6);
		assertOrder(v6, v7);
		assertOrder(v7, v8);

	}

}
